'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _babelPluginSyntaxFlow = require('babel-plugin-syntax-flow');

var _babelPluginSyntaxFlow2 = _interopRequireDefault(_babelPluginSyntaxFlow);

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

var _constantCase = require('constant-case');

var _constantCase2 = _interopRequireDefault(_constantCase);

var _babelAddFlowComments = require('babel-add-flow-comments');

var _babelAddFlowComments2 = _interopRequireDefault(_babelAddFlowComments);

var _s2sUtils = require('s2s-utils');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// import blog from 'babel-log'

const builders = {
  constants: (0, _s2sUtils.template)(`export const NAME: VALUE = VALUE`)
};

const prefix = process.env.S2S_ACTION_CONSTANTS_PREFIX ? `${process.env.S2S_ACTION_CONSTANTS_PREFIX}/` : '';

exports.default = () => {
  return {
    inherits: _babelPluginSyntaxFlow2.default,
    name: 's2s-d-action-constants',
    visitor: {
      Program: {
        exit(programPath) {
          const typeNameSet = new Set();
          const actionMap = new Map();

          function addTypes(path) {
            const v = path.get('id').node.name;
            typeNameSet.add(v);
          }

          programPath.traverse({
            TypeAlias(path) {
              if (path.get('id').node.name === 'Action') {
                const right = path.get('right');
                if (right.isUnionTypeAnnotation()) {
                  for (const typePath of right.get('types')) {
                    addTypes(typePath);
                  }
                } else if (right.isGenericTypeAnnotation()) {
                  addTypes(right);
                }
                // remove `type Action = ...`
                path.remove();
              } else {
                const node = path.get('id').node;
                actionMap.set(node.name, path.parentPath.node);
              }
            }
          });

          const typeNames = [...typeNameSet.values()];

          const constantAST = typeNames.map(x => {
            const name = (0, _constantCase2.default)(x);
            const value = t.stringLiteral(`${prefix}${name}`);

            const actionName = t.identifier(name);
            actionName.typeAnnotation = t.typeAnnotation(t.genericTypeAnnotation(value));

            return builders.constants({ NAME: actionName, VALUE: value });
          });

          programPath.node.body = [...constantAST];

          (0, _babelAddFlowComments2.default)(programPath);
        }
      }
    }
  };
};